#
# TWiki WikiClone (see TWiki.pm for $wikiversion and other info)
#
# Copyright (C) 1999-2001 Esteban Manchado, zoso@foton.es
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at 
# http://www.gnu.org/copyleft/gpl.html

# $Id: F-DEA30DC76ECD388426420AE1A3922485.pm,v 1.14 2003/06/10 11:32:30 zoso Exp $

package TWiki::Notifications;

use Text::Soundex;
use TWiki::Net;
use TWiki::Prefs;
use TWiki;

use strict;

use vars qw($notiftagregex $notifparamregex);

$notiftagregex = qr/[a-z]+/i;
$notifparamregex = qr/[a-z0-9@._-]+/i;

my $msgOrigTo = "Este mensaje iba dirigido originalmente a ";

my $msgTemplate =
'From: $from
To: $to
Subject: [N] $topic en $web de $sendername
Reply-To: "$sendername" <$from>

$extra$sendername le enva el siguiente texto del nodo
$web.$topic del TWiki$notifAlsoTo:


$text


--
https://laser.foton.es/twiki/bin/view/$web/$topic
';

# If you want the last diff attached to the notification, add this at the end
# of msgTemplate:
# -------------------------------- 8< --------------------------------
# ltimo diff (revisin $lastRev):
# ----------------------------------------------------------------------
# $lastDiff
# ----------------------------------------------------------------------
# -------------------------------- >8 --------------------------------

my $notifAlsoTo = ' (este mensaje tambin iba a: $recipsList)';



# =========================
# Notifies (currently by email) to recipients
sub notify
{
   my( $theWeb, $theTopic, $theText, $theSender, $theRecipientList ) = @_;

   my $domain = TWiki::Prefs::getPreferencesValue( 'SMTPSENDERHOST' ) || "localhost.localdomain";
   my $defaultAddress = TWiki::Prefs::getPreferencesValue( 'WIKIWEBMASTER' ) || "admin\@$domain";

   # first, decode recipient list
   $theRecipientList =~ s/^ *(.+?) *$/$1/;
   my @recipientlist = split( /[\s,]+/, $theRecipientList );

   # read user login list
   my @users;
   open( F, "$TWiki::dataDir/.htpasswd" ) or die "Can't open $TWiki::dataDir/.htpasswd";
   push @users, ( split ':' )[0] while ( <F> );
   close F;
   # add VALID_DOMAIN_ACCOUNTS contents to the user list
   my @valid_domain_accounts = split( ';', TWiki::Prefs::getPreferencesValue( 'VALID_DOMAIN_ACCOUNTS' ) );
   push @users, @valid_domain_accounts;

   my %alias;
   foreach my $pref ( split( ';',
                             TWiki::Prefs::getPreferencesValue( 'ALIAS' ) ) ) {
      my( $alias, $name ) = split( ':', $pref );
      $alias{$alias} = $name;
   }

   # get final recipient list
   my @real_recips;
   foreach my $r ( @recipientlist ) {
     my $user  = undef;

     # if it looks like a straight email address, use directly
     if( $r =~ /\@/ ) {
       push @real_recips, $r;
       next ;
     }

     # try to match alias
     foreach my $a ( keys %alias ) {
       if( $r eq $a ) {
         $user = $alias{$a};
         last ;
       }
     }

     if( not defined $user ) {
       # find if it's equal (or is pronounced like) any real username
       my $sound = soundex( $r );
       foreach my $u ( @users ) {
         if( $sound eq soundex( $u ) ) {
           $user = $u;
           last ;
         }
       }
     }
     push @real_recips, $user;    # perhaps undef
   }

   # get last diff (doesn't work anymore, because the topic isn't saved yet -- TODO)
   my $lastRev = TWiki::Store::getRevisionNumber($theWeb, $theTopic);
   $lastRev =~ s/1\.//;
   my $lastDiff = $lastRev <= 1 ? "" : TWiki::Store::getRevisionDiff($theWeb, $theTopic, "1." . ($lastRev - 1));

   # text recipient list
   my $recipsList;

   # process list
   for ( my $i = 0; $i <= $#real_recips; ++$i ) {
     my( $msg, $to, $from );
     my $extra = "";
     my ( $wikiUserName, $sendername );
     if( defined $real_recips[$i] ) {
         if ( $real_recips[$i] !~ /\@/ ) {
             $wikiUserName = &TWiki::userToWikiName( $real_recips[$i], 1 );
             $to = TWiki::Prefs::getPropertyValue( 'Email', 'Main', $wikiUserName );
             defined $to || do {
                if ( grep { $_ eq $real_recips[$i] }
                          @valid_domain_accounts ) {
                   $to = $real_recips[$i]."\@$domain";
                } else {
                   $to = $defaultAddress;
                   $extra = $msgOrigTo . "$wikiUserName ($recipientlist[$i])" .
                            "\n\n" . ( "-" x 70 ) . "\n\n";
                }
             };
         } else {
             $to = $real_recips[$i];
         }
     } else {
         $to = $defaultAddress;
         $extra = $msgOrigTo .  $recipientlist[$i] . "\n\n" .
                  ( "-" x 70 ) . "\n\n";
     }
     my @tmpList = @real_recips;
     splice( @tmpList, $i, 1 );
     @tmpList = grep { defined $_ } @tmpList;
     $recipsList = join( ', ', @tmpList );
     $msg = $msgTemplate;
     # get sender email address
     $wikiUserName = &TWiki::userToWikiName( $theSender, 1 );
     $from = TWiki::Prefs::getPropertyValue( 'Email', 'Main', $wikiUserName );
     $from ||= $defaultAddress;
     $sendername = TWiki::Prefs::getPropertyValue( 'Name', 'Main', $wikiUserName );
     # process template
     $msg =~ s/\$sendername\b/$sendername/g;
     $msg =~ s/\$from\b/$from/g;
     $msg =~ s/\$to\b/$to/g;
     $msg =~ s/\$web\b/$theWeb/g;
     $msg =~ s/\$topic\b/$theTopic/g;
     $msg =~ s/\$text\b/$theText/g;
     $msg =~ s/\$lastDiff\b/$lastDiff/g;
     $msg =~ s/\$lastRev/$lastRev/g;
     $msg =~ s/\$extra/$extra/g;
     my $notifAlsoToTmpl = "";
     if( $recipsList ) {
       $notifAlsoToTmpl = $notifAlsoTo;
       $notifAlsoToTmpl =~ s/\$recipsList/$recipsList/;
     }
     $msg =~ s/\$notifAlsoTo/$notifAlsoToTmpl/g;
     TWiki::Net::sendEmail( $msg );
   }
}

# =========================
# Identifies notification syntax
sub identify_notif
{
   my( $theNotificationParams ) = @_;

   return $theNotificationParams =~ /^\s*($notiftagregex\:)?\s*$notifparamregex([\s,]+$notifparamregex)*(\s*[,;]+\s*$notiftagregex\:\s*$notifparamregex([\s,]+$notifparamregex)*)*$/i;
}

# =========================
# Splits notification syntax into a proper structure
sub split_notif_params
{
   my( $theNotificationParams ) = @_;

   my( $initial, $result );
   $result = [];
   while ($theNotificationParams ne '') {
      $initial = $theNotificationParams;

      if ($theNotificationParams =~ /^\s*[,;]*\s*($notiftagregex):/i) {
         push @$result, { tag => $1, params => [] };
         $theNotificationParams =~ s/^\s*[,;]*\s*($notiftagregex)://i;
      }

      if ($theNotificationParams =~ /\s*($notifparamregex)/i) {
         scalar @$result || do { push @$result, { tag    => '_default',
                                                  params => [] } };
         push @{$result->[$#$result]->{params}}, $1;
         $theNotificationParams =~ s/\s*($notifparamregex)//i;
      }

      # Against potential infinite loops
      last if $initial eq $theNotificationParams;
   }
   return $result;
}

# =========================
# Makes the appropriate calls
sub notif_call
{
   my( $theNotificationCalls, $theText, $theWeb, $theTopic, $theUser ) = @_;

   foreach my $call (@$theNotificationCalls) {
      if ($call->{tag} eq '_default') {
         notify($theWeb, $theTopic, $theText,
                         $theUser,  join(' ', @{$call->{params}}));
      } else {
         (my $constname = lc($call->{tag})) =~ s/.*:://;
         $constname =~ s/.*/\u$&/;     # Normalize
         no strict qw(refs);
         eval "use TWiki::Notifications::$constname";
         my $sub = "TWiki::Notifications::$constname\::notify";
         eval { $theText = &$sub($call->{params}, $theText,  $theWeb,
                                                  $theTopic, $theUser); };
         if ($@) {
            $theText .= " [unknown tag ".$call->{tag}."]";
         }
      }
   }

   return $theText;
}

# =========================

1;

# EOF
