#!/usr/bin/perl -w

use lib qw(. /home/httpd/twiki/lib);

use strict;
use Test::More tests => 12;
use Test::Deep;

use TWiki::Notifications;

my $test_notif1 = "task: general 20, jabber: k, z";
my $test_notif2 = "general; j: k z";
my $test_notif3 = "general j: k z";             # Missing "," or ";"
my $test_notif4 = 'zoso k, zoso@debian.org';
my $test_notif5 = 'zoso';
my $test_notif6 = 'zoso; jander|klander';
my $test_notif7 = 'zoso_something, zoso-anything';
my $test_notif8 = 'tesT: one, two three';
ok (TWiki::Notifications::identify_notif($test_notif1), "identify_notif");
ok (TWiki::Notifications::identify_notif($test_notif2), " implicit");
ok (!TWiki::Notifications::identify_notif($test_notif3)," error");
ok (TWiki::Notifications::identify_notif($test_notif4), " simple");
ok (TWiki::Notifications::identify_notif($test_notif5), " very simple");
ok (!TWiki::Notifications::identify_notif($test_notif6)," another error");
ok (TWiki::Notifications::identify_notif($test_notif7), " underscores, dashes");
ok (TWiki::Notifications::identify_notif($test_notif8), " irregular case");

my $split_notif1 = [ { tag => 'task',   params => ['general', '20'] },
                     { tag => 'jabber', params => ['k', 'z']        } ];
my $split_notif2 = [ { tag => '_default', params => ['general'] },
                     { tag => 'j',        params => ['k', 'z']        } ];
my $split_notif4 = [ { tag => '_default', params => ['zoso', 'k',
                                                     'zoso@debian.org'] } ];
my $split_notif8 = [ { tag => 'tesT', params => ['one', 'two', 'three'] } ];
cmp_deeply (TWiki::Notifications::split_notif_params($test_notif1), $split_notif1, "split_notif_params");
cmp_deeply (TWiki::Notifications::split_notif_params($test_notif2), $split_notif2, " test 2");
cmp_deeply (TWiki::Notifications::split_notif_params($test_notif4), $split_notif4, " test 4");
cmp_deeply (TWiki::Notifications::split_notif_params($test_notif8), $split_notif8, " test 8");
